/**
* @FileName:     ImageManager.cpp
* @Modified:	 Brian Hinds
* @Date:         18-01-2017
* @Version:      1.0.2
*/

#include "SetTrekPlugins.h"

/// <summary>
/// Constructor for loading bitmaps from file
/// </summary>
/// <method name= "ImagaeManager"></method>
/// <param name="wchar_t * filename, Graphics * EngineGFX"></param>
/// <returns>Bitmap Image</returns>
ImageManager::ImageManager(wchar_t * filename, Graphics* EngineGFX)
{
	/// <summary>
	/// Setting up objects to process bitmap image
	/// </summary>
	this->EngineGFX = EngineGFX;
	GameBmpAsset = NULL;
	HRESULT GameAssetHR;

	/// <summary>
	/// Create factory needed for the bitmap image
	/// </summary>
	IWICImagingFactory *wicFactory = NULL;
	GameAssetHR = CoCreateInstance(
		CLSID_WICImagingFactory,
		NULL,
		CLSCTX_INPROC_SERVER,
		IID_IWICImagingFactory,
		(LPVOID*)&wicFactory);
	if (!GameAssetHR == S_OK)
	{
		MessageBox(0, "Error Creating IWICImagingFactory", "Image Manager Error", MB_OK);
	}

	/// <summary>
	/// Gets the bitmap image from file
	/// </summary>
	IWICBitmapDecoder *wicDecoder = NULL;
	GameAssetHR = wicFactory->CreateDecoderFromFilename(
		filename,
		NULL,
		GENERIC_READ,
		WICDecodeMetadataCacheOnLoad,
		&wicDecoder);
	if (!GameAssetHR == S_OK)
	{
		MessageBox(0, "Error reading image file or no such file exist", "Image Manager Error", MB_OK);
	}

	/// <summary>
	/// Read the image frame
	/// </summary>
	IWICBitmapFrameDecode* wicFrame = NULL;
	GameAssetHR = wicDecoder->GetFrame(0, &wicFrame);
	if (!GameAssetHR == S_OK)
	{
		MessageBox(0, "Error reading image frame ", "Image Manager Error", MB_OK);
	}

	/// <summary>
	/// Create a converter for the bitmap image
	/// </summary>
	IWICFormatConverter* wicConverter = NULL;
	GameAssetHR = wicFactory->CreateFormatConverter(&wicConverter);
	if (!GameAssetHR == S_OK)
	{
		MessageBox(0, "Error creating image converter", "Image Manager Error", MB_OK);
	}

	/// <summary>
	/// Setup the converter for the bitmap image
	/// </summary>
	GameAssetHR = wicConverter->Initialize(
		wicFrame,
		GUID_WICPixelFormat32bppPBGRA,
		WICBitmapDitherTypeNone,
		NULL,
		0.0,
		WICBitmapPaletteTypeCustom
	);
	if (!GameAssetHR == S_OK)
	{
		MessageBox(0, "Error converting image", "Image Manager Error", MB_OK);
	}

	/// <summary>
	/// Creates a D2D1Bitmap image
	/// </summary>
	EngineGFX->Rendertarget()->CreateBitmapFromWicBitmap(
		wicConverter,
		NULL,
		&GameBmpAsset
	);

	/// <summary>
	/// Releases the factory objects
	/// </summary>
	if (wicFactory) wicFactory->Release();
	if (wicDecoder) wicDecoder->Release();
	if (wicConverter) wicConverter->Release();
	if (wicFrame) wicFrame->Release();

	NPC_Width = GameBmpAsset->GetSize().width;
	NPC_Height = GameBmpAsset->GetSize().height;
	NPC_Accross = 1;

	ChomaKeyImg_Width = GameBmpAsset->GetSize().width;
	ChomaKeyImg_Height = GameBmpAsset->GetSize().height;
}

/// <summary>
/// Constructor for loading NPCs from file
/// </summary>
/// <method name= "ImagaeManager"></method>
/// <param name="wchar_t * filename, Graphics * EngineGFX, int NPC_Width, int NPC_Height"></param>
/// <returns>Bitmap Image</returns>
ImageManager::ImageManager(wchar_t * filename, Graphics * EngineGFX, int NPC_Width, int NPC_Height) :
	ImageManager(filename, EngineGFX)
{
	this->NPC_Height = NPC_Height;
	this->NPC_Width = NPC_Width;
	this->NPC_Accross = GameBmpAsset->GetSize().width / NPC_Width;
}

/// <summary>
/// Constructor for chroma-key bitmap from file
/// </summary>
/// <method name= "ImagaeManager"></method>
/// <param name="wchar_t * filename, Graphics * EngineGFX, int ChromaValue"></param>
/// <returns>Bitmap Image</returns>
ImageManager::ImageManager(wchar_t * filename, Graphics * EngineGFX, int ChromaValue) :
	ImageManager(filename, EngineGFX)
{
	D2D1_VECTOR_4F Vec4F = { 0.0f, 1.0f, 0.0f, 0.0f };
	m_d2dContext->CreateEffect(CLSID_D2D1ChromaKey, &chromakeyEffect);

	chromakeyEffect->SetInput(ChromaValue, GameBmpAsset);
	chromakeyEffect->SetValue(D2D1_CHROMAKEY_PROP_COLOR, Vec4F);
	chromakeyEffect->SetValue(D2D1_CHROMAKEY_PROP_TOLERANCE, 0.2f);
	chromakeyEffect->SetValue(D2D1_CHROMAKEY_PROP_INVERT_ALPHA, false);
	chromakeyEffect->SetValue(D2D1_CHROMAKEY_PROP_FEATHER, false);
}

/// <summary>
/// Destructor for bitmap
/// </summary>
/// <method name= "~ImageManager"></method>
/// <param name="None"></param>
/// <returns>None</returns>
ImageManager::~ImageManager()
{
	if (GameBmpAsset) GameBmpAsset->Release();
}

/// <summary>
/// Draws the bitmap image
/// </summary>
/// <method name= "Draw"></method>
/// <param name="None"></param>
/// <returns>void</returns>
void ImageManager::Draw(float X_Position, float Y_Position)
{
	EngineGFX->Rendertarget()->DrawBitmap(
		GameBmpAsset,
		D2D1::Rect(X_Position, Y_Position, GameBmpAsset->GetSize().width + X_Position, GameBmpAsset->GetSize().height + Y_Position),
		1.f,
		D2D1_BITMAP_INTERPOLATION_MODE::D2D1_BITMAP_INTERPOLATION_MODE_NEAREST_NEIGHBOR,
		D2D1::Rect(0.0f, 0.0f, GameBmpAsset->GetSize().width, GameBmpAsset->GetSize().height)
	);
}

/// <summary>
/// Draws animated none playable characters
/// </summary>
/// <method name= "DrawNPC"></method>
/// <param name="int index, float X_Position, float Y_Position"></param>
/// <returns>void</returns>
void ImageManager::DrawNPC(int index, float X_Position, float Y_Position)
{
	D2D_RECT_F src = D2D1::RectF(
		(float)((index % NPC_Accross)*NPC_Width),
		(float)((index / NPC_Accross)*NPC_Height),
		(float)((index % NPC_Accross)*NPC_Width) + NPC_Width,
		(float)((index / NPC_Accross)*NPC_Height) + NPC_Height);

	D2D_RECT_F dest = D2D1::RectF(
		X_Position, Y_Position, X_Position + NPC_Width, Y_Position + NPC_Height);

	EngineGFX->Rendertarget()->DrawBitmap(
		GameBmpAsset,
		dest,
		1.0f,
		D2D1_BITMAP_INTERPOLATION_MODE::D2D1_BITMAP_INTERPOLATION_MODE_NEAREST_NEIGHBOR,
		src);
}

/// <summary>
/// Draws chroma-keyed assets
/// </summary>
/// <method name= "DrawChromaKeyedImage"></method>
/// <param name="int index, float X_Position, float Y_Position"></param>
/// <returns>void</returns>
void ImageManager::DrawChromaKeyedImage(float X_Position, float Y_Position)
{
	m_d2dContext->DrawImage(chromakeyEffect.Get());
	D2D_RECT_F dest = D2D1::RectF(X_Position, Y_Position, X_Position + ChomaKeyImg_Width, Y_Position + ChomaKeyImg_Height);

	/*m_d2dContext->DrawImage(chromakeyEffect.Get(), dest);*/
}